<?php

class DbOperations{

    private $con;

    function __construct(){
        require_once dirname(__FILE__) . '/DbConnect.php';
        $db = new DbConnect;
        $this->con = $db->connect();
    }


    public function createUser($firstName, $lastName, $email, $idCode, $password, $phoneNumber, $address, $state, $country){
        if(!$this->isIdCodeExist($idCode)){
            $stmt = $this->con->prepare("INSERT INTO staff_register (firstName,	lastName, email, idCode, password, phoneNumber, address, state, country) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)");
            $stmt->bind_param("sssssssss", $firstName, $lastName, $email, $idCode, $password, $phoneNumber, $address, $state, $country);
            if($stmt->execute()){
                return USER_CREATED;
            }else{
                return USER_FAILURE;
            
        }
            return USER_EXISTS;
        }
    }

    // --------------------------------------------------------------------------------------------

    public function userLogin($idCode, $password){
        if($this->isIdCodeExist($idCode)){
            $hashed_password = $this->getUserPasswordByIdCode($idCode);
            if(password_verify($password, $hashed_password)){
                return USER_AUTHENTICATED;
            }else{
                return USER_PASSWORD_DO_NOT_MATCH;
            }

        }else{
            return USER_NOT_FOUND;
        }
    }

// -----------------------------------------------------------------------------------------------

private function getUserPasswordByIdCode($idCode){
    $stmt = $this->con->prepare("SELECT password FROM  staff_register WHERE idCode = ?");
    $stmt->bind_param("s", $idCode);
    $stmt->execute();
    $stmt->bind_result($password);
    $stmt->fetch();
  return $password;
}



    // --------------------------------------------------------------------------------------------

    public function getAllUsers(){
        $stmt = $this->con->prepare("SELECT id, firstName, lastName, email, idCode, phoneNumber, address, state, country FROM  staff_register;");
        $stmt->execute();
        $stmt->bind_result($id, $firstName, $lastName, $email, $idCode, $phoneNumber, $address, $state, $country);
        $users = array();
        while($stmt->fetch()){
        $user = array();
        $user['id'] = $id;
        $user['firstName']=$firstName;
        $user['lastName'] = $lastName;
        $user['email'] = $email;
        $user['idCode'] = $idCode;
        $user['phoneNumber']=$phoneNumber;
        $user['address']=$address;
        $user['state'] = $state;
        $user['country'] = $country;
        array_push($users, $user);
        }
        return $users;
    }


    // ----------------------------------------------------------------------------------------------


    public function getUserByIdCode($idCode){
        $stmt = $this->con->prepare("SELECT id, firstName, lastName, email, idCode, phoneNumber, address, state, country FROM  staff_register WHERE idCode = ?");
        $stmt->bind_param("s", $idCode);
        $stmt->execute();
        $stmt->bind_result($id, $firstName, $lastName, $email, $idCode, $phoneNumber, $address, $state, $country);
        $stmt->fetch();
        $user = array();
        $user['id'] = $id;
        $user['firstName']=$firstName;
        $user['lastName'] = $lastName;
        $user['email'] = $email;
        $user['idCode'] = $idCode;
        $user['phoneNumber']=$phoneNumber;
        $user['address']=$address;
        $user['state'] = $state;
        $user['country'] = $country;
        return $user;
    }

    // ------------------------------------------------------------------------------------------------------

    public function updateUsers($firstName, $lastName, $email, $idCode, $phoneNumber, $address, $state, $country, $id){
        $stmt = $this->con->prepare("UPDATE staff_register SET firstName = ?, lastName = ?, email = ?, idCode = ?, phoneNumber = ?, address = ?, state = ?, country = ? WHERE id = ?");
        $stmt->bind_param("ssssssssi", $firstName, $lastName, $email, $idCode, $phoneNumber, $address, $state, $country, $id);
        if($stmt->execute())
            return true;
        return false;
    }

    // ---------------------------------------------------------------------------------------------------------

    public function updatePassword($currentpassword, $newpassword, $idCode){
        $hashed_password = $this->getUserPasswordByIdCode($idCode);

        if(password_verify($currentpassword, $hashed_password)){

            $hash_password = password_hash($newpassword, PASSWORD_DEFAULT);
            $stmt = $this->con->prepare("UPDATE staff_register SET password = ? WHERE idCode = ?");
            $stmt->bind_param("ss", $hash_password, $idCode);

            if($stmt->execute())
                return PASSWORD_CHANGED;
            return PASSWORD_NOT_CHANGED;
        
        }else{
            return PASSWORD_DO_NOT_MATCH;
        }
    }

    // -----------------------------------------------------------------------------------------------------

    public function deleteUser($id){
        $stmt = $this->con->prepare("DELETE FROM staff_register WHERE id = ?");
        $stmt->bind_param("i", $id);
        if($stmt->execute())
            return true;
        return false;
    }




    // ----------------------------------------------------------------------------------------------

    private function isIdCodeExist($idCode){
        $stmt = $this->con->prepare("SELECT id  FROM  staff_register WHERE idCode = ?");
        $stmt->bind_param("s", $idCode);
        $stmt->execute();
        $stmt->store_result();
        return $stmt->num_rows > 0;
    }

}


// --------------------------------------------------------------------------------------------------------
//  ????????????????????????????????????????????????????????????????????????????????????????????????????????

// NEW CLASS FOR VERIFICATION

// ---------------------------------------------------------------------------------------------------------

class Verification{

    private $con;

    function __construct(){
        require_once dirname(__FILE__) . '/DbConnect.php';
        $db = new DbConnect;
        $this->con = $db->connect();
    }



    // For POST Requst

    public function uploadImage($image, $names, $idCode){
     
        if(!$this->isIdCodeExist($idCode)){
            
            //  $tempname = $_FILES['image']['tmp_name'];
            // $uploads = 'uploads/'.$file;
            // move_uploaded_file($tempname, $uploads);

            $target_dir = "uploads/";
          //  $file = addslashes(file_get_contents($_FILES["image"]["tmp_name"]));
            $file = $target_dir.basename($_FILES["image"]["name"]);
            move_uploaded_file($_FILES["image"]["tmp_name"], $target_dir.$_FILES["image"]["name"]);
            

            $stmt = $this->con->prepare("INSERT INTO staff_verification (image,	names, idCode) VALUES (?, ?, ?)");
            $stmt->bind_param("sss", $file, $names, $idCode);

           // $stmt = move_uploaded_file($tempname, $uploads);

            if($stmt->execute()){
                return IMAGE_UPLOADED;
            }else{
                return IMAGE_UPLOAD_FAILED;
            
        }
            
        }
        return ID_EXISTS;
    }





    private function isIdCodeExist($idCode){
        $stmt = $this->con->prepare("SELECT id  FROM  staff_verification WHERE idCode = ?");
        $stmt->bind_param("s", $idCode);
        $stmt->execute();
        $stmt->store_result();
        return $stmt->num_rows > 0;
    }

}



// ?????????????????????????????????????????????????????????????????????????????????????????????????????????
// ----------------------------------------------------------------------------------------------------------