<?php

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use Slim\Factory\AppFactory;

//  require __DIR__ . '/../vendor/autoload.php';

//  require __DIR__ . '/../includes/DbOperations.php';

 require  '../vendor/autoload.php';

 require '../Includes/DbOperations.php';


$app = new \Slim\App([
    'settings'=>[
        'displayErrorDetails'=>true
    ]
]);




/*
                FIRST CLASS
                
    endpoint: createuser
    parameters: firstName,	lastName,	email,	password,	phoneNumber,	address, state,	country
    method: POST
*/


$app->post('/createuser', function(Request $request, Response $response){

    if(!haveEmptyParameters(array('firstName', 'lastName', 'email', 'idCode', 'password', 'phoneNumber', 'address', 'state', 'country'), $request, $response)){
        $request_data = $request->getParsedBody();
        $firstName = $request_data['firstName'];
        $lastName = $request_data['lastName'];
        $email = $request_data['email'];
        $idCode = $request_data['idCode'];
        $password = $request_data['password'];
        $phoneNumber = $request_data['phoneNumber'];
        $address = $request_data['address'];
        $state = $request_data['state'];
        $country = $request_data['country'];
        $hash_password = password_hash($password, PASSWORD_DEFAULT);

        $db = new DbOperations;

        $result = $db->createUser($firstName, $lastName, $email, $idCode, $hash_password, $phoneNumber, $address, $state, $country);

        if($result == USER_CREATED){

            $message = array();
            $message['error'] = false;
            $message['message'] = 'User created successfully';

            $response->write(json_encode($message));

            return $response
                        ->withHeader('content-type', 'application/json')
                        ->withStatus(201);

        }else if($result == USER_FAILURE){

            $message = array();
            $message['error'] = true;
            $message['message'] = 'Some error occurred';

            $response->write(json_encode($message));

            return $response
                        ->withHeader('content-type', 'application/json')
                        ->withStatus(422);

        }else if($result == USER_EXISTS){
            $message = array();
            $message['error'] = true;
            $message['message'] = 'User Already Exist';

            $response->write(json_encode($message));

            return $response
                        ->withHeader('content-type', 'application/json')
                        ->withStatus(422);
        }
    }
    return $response
         ->withHeader('content-type', 'application/json')
         ->withStatus(422);
});


// ------------------------------------------------------------------------------------------------------



$app->post('/userlogin', function(Request $request, Response $response){

    if(!haveEmptyParameters(array('idCode', 'password'), $request, $response)){
        $request_data = $request->getParsedBody();

        $idCode = $request_data['idCode'];
        $password = $request_data['password'];

        $db = new DbOperations;

        $result = $db->userLogin($idCode, $password);

        if($result == USER_AUTHENTICATED){

            $user = $db->getUserByIdCode($idCode);
            $response_data = array();

            $response_data['error']=false;
            $response_data['message'] = 'Login Successful';
            $response_data['user']=$user;

            $response->write(json_encode($response_data));

            return $response
                ->withHeader('content-type', 'application/json')
                ->withStatus(200);

        }else if($result == USER_NOT_FOUND){
            $response_data = array();

            $response_data['error']=true;
            $response_data['message']='User not exist';

            $response->write(json_encode($response_data));

            return $response
                ->withHeader('content-type', 'application/json')
                ->withStatus(200);

        }else if($result == USER_PASSWORD_DO_NOT_MATCH){
            $response_data = array();

            $response_data['error']=true;
            $response_data['message']='Invalid credential';

            $response->write(json_encode($response_data));

            return $response
                ->withHeader('content-type', 'application/json')
                ->withStatus(200);

        }
    }
    return $response
        ->withHeader('content-type', 'application/json')
        ->withStatus(422);

});


// -----------------------------------------------------------------

$app->get('/allusers', function(Request $request, Response $response){

    $db = new DbOperations;

    $users = $db->getAllUsers();

    $response_data = array();

    $response_data['error'] = false;
    $response_data['users'] = $users;

    $response->write(json_encode($response_data));

    return $response
        ->withHeader('content-type', 'application/json')
        ->withStatus(200);
});

// ------------------------------------------------------------------------------------



//??????????????????????????????????????????????????????????????????????????????????????


$app->get('/getuser/{idCode}', function(Request $request, Response $response, array $args){
    
    $idCode = $args['idCode'];

    $db = new DbOperations;

    $users = $db->getUserByIdCode($idCode);

    $response_data = array();

    $response_data['error'] = false;
    $response_data['users'] = $users;

    $response->write(json_encode($response_data));

    return $response
        ->withHeader('content-type', 'application/json')
        ->withStatus(200);
});


// ?????????????????????????????????????????????????????????????????????????????????????


// ----------------------------------------------------------------------

$app->put('/updateusers/{id}', function(Request $request, Response $response, array $args){

    $id = $args['id'];

    if(!haveEmptyParameters(array('firstName', 'lastName', 'email', 'idCode', 'phoneNumber', 'address', 'state', 'country'), $request, $response)){

        $request_data = $request->getParsedBody();
        $firstName = $request_data['firstName'];
        $lastName = $request_data['lastName'];
        $email = $request_data['email'];
        $idCode = $request_data['idCode'];
        $phoneNumber = $request_data['phoneNumber'];
        $address = $request_data['address'];
        $state = $request_data['state'];
        $country = $request_data['country'];
        $id = $request_data['id'];

        $db = new DbOperations;

        if($db->updateUsers($firstName, $lastName, $email, $idCode, $phoneNumber, $address, $state, $country, $id)){
            $response_data = array();
            $response_data['error'] = false;
            $response_data['message'] = 'User Update Successfully';
            $user = $db->getUserByidCode($idCode);
            $response_data['user'] = $user;

            $response->write(json_encode($response_data));

            return $response
                ->withHeader('content-type', 'application/json')
                ->withStatus(200);
        }else{
            $response_data = array();
            $response_data['error'] = true;
            $response_data['message'] = 'Please try again later';
            $user = $db->getUserByIdCode($idCode);
            $response_data['user'] = $user;

            $response->write(json_encode($response_data));

            return $response
                ->withHeader('content-type', 'application/json')
                ->withStatus(200);
        }
    }
    return $response
        ->withHeader('content-type', 'application/json')
        ->withStatus(200);
});

// ---------------------------------------------------------------

$app->put('/updatepassword', function(Request $request, Response $response){

    if(!haveEmptyParameters(array('currentpassword', 'newpassword', 'idCode'),  $request, $response)){

        $request_data = $request->getParsedBody();

        $currentpassword = $request_data['currentpassword'];
        $newpassword = $request_data['newpassword'];
        $idCode = $request_data['idCode'];

        $db = new DbOperations;

        $result = $db->updatePassword($currentpassword, $newpassword, $idCode);

        if($result == PASSWORD_CHANGED){
            $response_data = array();
            $response_data['error'] = false;
            $response_data['message'] = 'Password Changed';
            $response->write(json_encode($response_data));
            return $response->withHeader('content-type', 'application/json')
                            ->withStatus(200);
        }else if($result == PASSWORD_DO_NOT_MATCH){
            $response_data = array();
            $response_data['error'] = true;
            $response_data['message'] = 'You have given wrong password';
            $response->write(json_encode($response_data));
            return $response->withHeader('content-type', 'application/json')
                            ->withStatus(200);
        }else if($result == PASSWORD_NOT_CHANGED){
            $response_data = array();
            $response_data['error'] = true;
            $response_data['message'] = 'Some error occured';
            $response->write(json_encode($response_data));
            return $response->withHeader('content-type', 'application/json')
                            ->withStatus(200);
        }
    }
    return $response
        ->withHeader('content-type', 'application/json')
        ->withStatus(422);

});

// -----------------------------------------------------------

$app->delete('/deleteuser/{id}', function(Request $request, Response $response, array $args){
    $id = $args['id'];

    $db = new DbOperations;

    if($db->deleteUser($id)){
        $response_data['error'] = false;
        $response_data['message'] = 'User has been deleted';
    }else{
        $response_data['error'] = true;
        $response_data['message'] = 'Please try again later';
    }

    $response->write(json_encode($response_data));
    
return $response
        ->withHeader('content-type', 'application/json')
        ->withStatus(200);
});

// --------------------------------------------------------



// ---------------------------------------------------------------------------------------------------------

// UPLOAD IMAGE FOR VERIFICATION

// ----------------------------------------------------------------------------------------------------------

$app->post('/uploadimage', function(Request $request, Response $response){

    if(!haveEmptyParameters(array('image', 'names', 'idCode'), $request, $response)){
       
        $request_data = $request->getParsedBody();
        
        $image = $request_data['image'];
        $names = $request_data['names'];
        $idCode = $request_data['idCode'];

        $db = new Verification;

        $result = $db->uploadImage($image, $names, $idCode);

        if($result == IMAGE_UPLOADED){

            $message = array();
            $message['error'] = false;
            $message['message'] = 'Image uploaded successfully';
           // $message['image'] = getBaseURL() . $_FILES;


            $response->write(json_encode($message));

            return $response
                        ->withHeader('content-type', 'application/json')
                        ->withStatus(201);

        }else if($result == IMAGE_UPLOAD_FAILED){

            $message = array();
            $message['error'] = true;
            $message['message'] = 'Some error occurred';

            $response->write(json_encode($message));

            return $response
                        ->withHeader('content-type', 'application/json')
                        ->withStatus(422);

        }else if($result == ID_EXISTS){
            $message = array();
            $message['error'] = true;
            $message['message'] = 'User Already Exist';

            $response->write(json_encode($message));

            return $response
                        ->withHeader('content-type', 'application/json')
                        ->withStatus(422);
        }
    }


    // function getBaseURL(){
    //     $url = isset($_SERVER[HTTPS]) ? 'https://' : 'http://';
    //     $url .= $_SERVER['SERVER_NAME'];
    //     $url .= $_SERVER['REQUEST_URI'];
    //     return dirname($url) . '/';
    // }

    return $response
         ->withHeader('content-type', 'application/json')
         ->withStatus(422);
});


// ------------------------------------------------------------------------------------------------------


// ---------------------------------------------------------

// Final Codes Constant

// ------------------------------------------------------

function haveEmptyParameters($required_params, $request, $response){
    $error = false;
    $error_params = '' ;
    $request_params = $request->getParsedBody();

    foreach($required_params as $param){
        if(!isset($request_params[$param]) || strlen($request_params[$param])<=0){
            $error = true;
            $error_params .= $param . ', ';
        }
    }

    if($error){
        $error_detail = array();
        $error_detail['error'] = true;
        $error_detail['message'] = 'Required parameters ' . substr($error_params, 0, -2) . ' are missing or empty';
        $response->write(json_encode($error_detail));
    }
    return $error;
}




$app->run();